@@ -81,7 +81,7 @@ var populate_kvs_from_object = function(branch, prefix, obj, cb) {
 
   render_obj(writes, prefix, obj);
 
-  logger.debug('JSON file %s yielded %s keys', prefix, writes.length);
+  logger.debug('Expandable file %s yielded %s keys', prefix, writes.length);
 
   cb = _.after(writes.length, cb);
 
@@ -90,7 +90,6 @@ var populate_kvs_from_object = function(branch, prefix, obj, cb) {
   });
 };
 
-
 /**
  * If a file was modified, read its new value and update consul's KV store.
  */
@@ -223,8 +222,8 @@ var process_records = function(branch, records, cb) {
     --pending_records;
 
     // If there are no pending records, callback with all errors seen, if any.
-    if (pending_records === 0) {
-      cb((errors_seen.length > 0) ? errors_seen : null);
+    if (pending_records <= 0) {
+      _.once(cb((errors_seen.length > 0) ? errors_seen : null));
     }
 
     // TODO: Add a watchdog timer?  It's a bit scary that this method may never fire its callback
@@ -246,17 +245,25 @@ var process_records = function(branch, records, cb) {
       case 'A':
       case 'T':
         // Store added/modified file
+        // FIXME: This will definitely fail in scenarios where record path is not in branch source root.
         if (record.path === branch.common_properties) {
-          branch.listAdditionalPropertyFiles(records, function(err, additionRecords) {
-            process_records(branch, additionRecords, function(errs) {
-              if (errs) {
-                return cb("Some consul updates failed:\n" + errs.join('\n'));
-              }
+          ++pending_records;
+
+          file_modified(branch, record.path, function(err) {
+            branch.listAdditionalPropertyFiles(records, function(err, additionalRecords) {
+              if (err) return check_pending(err);
+              if (additionalRecords.length == 0) return check_pending();
+              process_records(branch, additionalRecords, function(errs) {
+                if (errs) check_pending("Some consul updates failed:\n" + errs.join('\n'));
+
+                check_pending();
+              });
             });
           });
+        } else {
+          ++pending_records;
+          file_modified(branch, record.path, check_pending);
         }
-        ++pending_records;
-        file_modified(branch, record.path, check_pending);
         break;
       case 'D':
         // Delete file
